#
# Authors: Tom von Schwerdtner <tvon@etria.com>
#          Brian Skahan <bskahan@etria.com>
#
# Copyright 2004-2005, Etria, LLP
#
# This file is part of Quills
#
# Quills is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Quills is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Quills; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from OFS.SimpleItem import SimpleItem
from Products.CMFCore.utils import getToolByName
from Products.CMFDefault.utils import parseHeadersBody
from AccessControl import ClassSecurityInfo
import DateTime

authMethods = [
    'metaWeblog/getPost',
    'metaWeblog/deletePost',
    'metaWeblog/editPost',
    'metaWeblog/newPost',
    'metaWeblog/getRecentPosts',
    'metaWeblog/getUsersBlogs',
    'metaWeblog/getCategories'
    ]

def genericMetaWeblogAuth(args):
    return args[1], args[2], args


class MetaWeblogAPI(SimpleItem):
    """http://www.xmlrpc.com/metaWeblogApi"""
    
    security = ClassSecurityInfo()
    
    def __init__(self, RPCAuth = None):
        if RPCAuth:
            self.setupRPCAuth(RPCAuth)
    
    security.declarePublic('setupRPCAuth')
    def setupRPCAuth(self, RPCAuth):
        RPCAuth.addAuthProvider(authMethods, genericMetaWeblogAuth)
    
    security.declarePublic('newPost')
    def newPost(self, blogid, username, password, struct, publish):
        """Create a new entry - either in draft folder or directly in archive."""
        self.plone_log('metaWeblog/newPost')
        
        # gathering the tools
        qtool = getToolByName(self, 'quills_tool')
        weblog = qtool.getByUID(blogid)
        archive = getattr(weblog, 'archive')
        # We just store draft weblogentries in the weblog instance now.
        drafts = weblog

        # preparing the ingredients:
        body  = struct.get('description', struct.get('Description'))
        title = struct.get('title', struct.get('Title'))
        categories = struct.get('categories', struct.get('Categories'))
        effective_date = DateTime.DateTime()
        id = qtool.idFromTitle(title=title, metatype="WeblogEntry", weblog=weblog)

        # let's get cooking:
        
        # 2005-12-13 tomster: 
        # FIXME: nasty hack due to bug in workflow script: if we publish. we create
        # the new entry directly inside the archive (instead of in the drafts folder), if not, 
        # we create it inside the drafts folder - this is done to avoid calling the 'publish' 
        # transition which attempts to move the entry from the drafts folder into the archive
        # (which fails with a CopyError) but rather the 'publish_in_place'.
        if publish:
            try:
                dest = archive.createPath(effective_date)
            except Exception, e:
                self.plone_log("Cannot create destination: %s" % e)
                return False
        else:   
            dest = drafts
        
        try:
            dest.invokeFactory(id=id, type_name='WeblogEntry', title=title)
        except Exception, e:
            self.plone_log("Cannot create WeblogEntry: %s" % e)
            return False
        
        entry = getattr(dest, id)
        entry.setText(body, mimetype='text/html')
        entry.setSubject(categories)
        entry_uid = entry.UID()
        
        if publish:
            wf_tool = getToolByName(self, 'portal_workflow')
            entry.setEffectiveDate(effective_date)
            wf_tool.doActionFor(entry, 'publish_in_place', 'quills_workflow')
        
        self.plone_log("returning " + entry_uid)
        return entry_uid

    security.declarePublic('editPost')
    def editPost(self, postid, username, password, struct, publish):
        """Implement MetaWeblog editPost"""
        
        self.plone_log('metaWeblog/editPost')
        
        qtool = getToolByName(self, 'quills_tool')
        
        entry = qtool.getByUID(postid)
        
        body  = struct.get('description', struct.get('Description'))
        title = struct.get('title', struct.get('Title'))
        categories = struct.get('categories', struct.get('Categories'))

        entry.setText(body, mimetype='text/html')
        entry.setTitle(title)
        entry.setSubject(categories)

        # 2005-12-13 tomster:
        # for editing posts to work, we currently must avoid calling the publish workflow.

        #if publish:
        #   wf_tool = getToolByName(self, 'portal_workflow')
        #   entry.setEffectiveDate(DateTime.DateTime())
        #   wf_tool.doActionFor(entry, 'publish', 'quills_workflow')
        
        return True

    security.declarePublic('getPost')
    def getPost(self, postid, username, password):
        "Return a post as struct"
        
        self.plone_log('metaWeblog/getPost')
        
        qtool = getToolByName(self, 'quills_tool')
        obj = qtool.getByUID(postid)

        return self.entryStruct(obj)
    
    security.declarePublic('getCategories')
    def getCategories(self, blogid, username, password):
        """Returns a struct containing description, htmlUrl and rssUrl"""
        self.plone_log('metaWeblog/getCategories')
        
        qtool = getToolByName(self, 'quills_tool')
        
        weblog = qtool.getByUID(blogid)
        
        topics = weblog.getCategories()
        
        # 2005-12-13 tomster: 
        # this is kind of ugly: according to the RFC we should return an array
        # of structs, but according to http://typo.leetsoft.com/trac/ticket/256
        # (at least) MarsEdit and ecto expect an array of strings containing the
        # category id. Since those are the only two clients available to me, this
        # is what I implement. The original (RFC-conforming code is left commented
        # out.)
        categories = [topic.getId() for topic in topics]
        
        #for topic in topics:
        #    categories.append(
        #        {'description' : topic.id,
        #         'htmlUrl' : topic.absolute_url(),
        #         'rssUrl' : topic.absolute_url()
        #        })
        
        return categories
    
    security.declarePublic('deletePost')
    def deletePost(self, postid, username, password, publish):
        "Returns true on success, fault on failure"
        self.plone_log('metaWeblog/deletePost')
        
        qtool = getToolByName(self, 'quills_tool')
        
        entry = qtool.getByUID(postid)
        
        entry.aq_inner.aq_parent.manage_delObjects(entry.getId())
        
        return True
    
    security.declarePublic('getRecentPosts')
    def getRecentPosts(self, blogid, username, password, num):
        """Return 'num' recent posts to specified blog,
        
        returns a struct: The three basic elements are title, link and
        description. For blogging tools that don't support titles and links,
        the description element holds what the Blogger API refers to as
        'content'.
        """
        # gathering the tools
        qtool = getToolByName(self, 'quills_tool')
        weblog = qtool.getByUID(blogid)

        self.plone_log('metaWeblog/getRecentPosts')
        
        catalog = getToolByName(self, 'portal_catalog')
        results = catalog(meta_type='WeblogEntry',
                          path='/'.join(weblog.getPhysicalPath()),
                          sort_on='effective',
                          sort_order='reverse')
        
        posts = []
        for item in results[:num]:
            obj = item.getObject()
            posts.append(self.entryStruct(obj))
        return posts
    
    security.declarePublic('getUsersBlogs')
    def getUsersBlogs(self, username, password):
        """TODO: getUsersBlogs"""
        
        self.plone_log('metaWeblog/getUsersBlogs')
        
        catalog = getToolByName(self, 'portal_catalog')
        
        results = catalog(meta_type='Weblog', Creator=username)
        
        blogs = []
        for item in results:
            o = item.getObject()
            blogs.append(
                    {'url': o.absolute_url(),
                     'blogid' : o.UID(),
                     'blogName' : o.title_or_id()}
                    )
        
        return blogs
    
    security.declarePublic('getUserInfo')
    def getUserInfo(self, appkey, username, password):
        """Returns returns a struct containing userinfo
        
        userid, firstname, lastname, nickname, email, and url.
        """
        
        self.plone_log('metaWeblog/getUserInfo')
        
        membership=getToolByName(self, 'portal_membership')
        info={'name':'no name','email':'no email','userid':'no user id'
               ,'firstname':'no first name','lastname':'no last name'
               ,'url':'no url'}
        member=membership.getAuthenticatedMember()
        if member:
            for key,value in info.items():
                info[key] = getattr(member,key,None) or value
        return info
    
    def entryStruct(self, obj):
        """ returns the entry as a struct. Called by getRecentPosts and getPost."""
        return {'postid': obj.UID(),
                'dateCreated': obj.EffectiveDate(),
                'title': obj.Title(),
                'description' : "<div>" + obj.getText() + "</div>",
                'categories' : obj.getEntryCategories(),
                'link' : obj.absolute_url()
               }
