from zope.interface import Interface


class IEnclosure(Interface):
    """Represents an 'enclosed' file that is explicitly linked to within
    and IFeedEntry.

    This is here to support podcasting.
    """

    def getURL():
       """URL of the enclosed file.
       """

    def getLength():
       """Return the size/length of the enclosed file.
       """

    def __len__():
       """Synonym for getLength
       """

    def getMajorType():
       """Return the major content-type of the enclosed file.

           e.g. content-type = 'text/plain' would return 'text'.
       """

    def getMinorType():
       """Return the minor content-type of the enclosed file.

           e.g. content-type = 'text/plain' would return 'plain'.
       """

    def getType():
       """Return the content-type of the enclosed file.
       """


class IFeedEntry(Interface):
    """A single syndication feed entry.
    """

    def getWebURL():
        """URL for the web view of this IFeedEntry.
        """

    def getTitle():
        """Title of this entry.
        """

    def getDescription():
        """Description of this entry.
        """

    def getBody():
        """The raw body content of this entry.
        """

    def getUID():
        """Unique ID for this entry.
        """

    def getAuthor():
        """Author of this entry.
        """

    def getEffectiveDate():
        """The datetime this entry was first published.
        """

    def getModifiedDate():
        """The datetime this entry was last modified.
        """

    def getTags():
        """The tags/keywords/subjects associated with this entry.
        """

    def getRights():
        """The dublin core 'rights' associated with this entry.
        """

    def getEnclosure():
        """Return an IEnclosure instance or None.
        """


class IFeedSource(Interface):
    """A source of IFeedEntry objects for a feed.
    """

    def getFeedEntries():
        """A sequence of IFeedEntry objects.
        """


class IFeed(Interface):
    """A syndication feed e.g. RSS, atom, etc.
    """

    def getBaseURL():
        """Used for supporting relative URLs in feeds.
        """

    def getUpdatePeriod():
        """
        """

    def getUpdateFrequency():
        """
        """

    def getImageURL():
        """URL of an image that can be embedded in feeds.
        """

    def getEncoding():
        """Character encoding for the feed.
        """

    def getTitle():
        """The title of this feed.
        """

    def getDescription():
        """A description of the feed.
        """

    def getMaxEntries():
        """Maximum number of IFeedEntries to show for this feed.
        """

    def getUID():
        """Unique ID for this feed.
        """

    def getWebURL():
        """URL for the web view of this feed.
        """

    def getFeedSources():
        """A sequence of IFeedSource objects.
        """

    def getFeedEntries():
        """Return a sequence of IFeedEntry objects with which to build a feed.
        """

    def getSortedFeedEntries(feed_entries=None):
        """Return a sorted sequence of IFeedEntries.  If feed_entries is None,
        call getFeedEntries first.
        Sorting based on publication datetime, newest first.
        """

    def getModifiedDate():
        """The last modified datetime that applies to the whole feed.
        """


class ISyndicationManager(Interface):
    """Holds IFeed instances that are (optionally) associated with a particular
    content object.
    """

    def getAvailableFeeds():
        """A sequence of available IFeed objects for the associated
        content.
        """

    def getDefaultFeed():
        """
        """

    def setDefaultFeed(feed_id):
        """
        """


# Should probably inherit from IFolder
class ISyndicationTool(Interface):
    """Tool for managing syndication feeds for portal content.
    """

    def getSyndicationManagerFor(content):
        """Return the associated ISyndicationManager for the content object, or None.
        """

    def returnSyndicationManagerFor(content):
        """Guarantees to return a SyndicationManager, even if one doesn't yet exist.
        """

    def addSyndicationManagerFor(content):
        """Add a SyndicationManager (XXX Of which type?) for the content and return it.
        """
