# Standard library imports
import random

# Zope3 imports
from zope.interface import implements

# Zope2 imports
from AccessControl import ClassSecurityInfo
from DateTime.DateTime import DateTime

# CMF imports
from Products.CMFCore.utils import getToolByName

# Archetypes imports
from Products.Archetypes.public import BaseFolder, registerType

# Product imports
from schema import syndication_feed_schema
from config import PROJECTNAME, FEED_PORTAL_TYPE, \
    REFERENCED_FEED_SOURCE_TYPE, REFERENCED_COMMENT_FEED_SOURCE_TYPE
import permissions as perms

# basesyndication imports
from Products.basesyndication.interfaces import IFeed, IFeedSource


class FeedMixin:
    """Feed mixin class that can be used both for persistent SyndicationFeed
    objects that live in the syndication tool, and IFeed adapters.
    Consequently, methods must be able to work with 'context' that could be
    'self' (for SyndicationFeed objects) or 'self.context' (for adapters).
    """

    def getFeedEntries(self):
        """See IFeed.
        """
        entries = []
        for fs in self.getFeedSources():
            entries.extend(fs.getFeedEntries())
        return entries

    def getSortedFeedEntries(self, feed_entries=None):
        """Return a sorted sequence of IFeedEntries.  If feed_entries is None,
        call getFeedEntries first.
        Sorting based on publication datetime, newest first.
        """
        if feed_entries is None:
            feed_entries = self.getFeedEntries()
        feed_entries.sort( lambda x, y: cmp(y.getEffectiveDate(), x.getEffectiveDate()) )
        return feed_entries

    def getUpdatePeriod(self):
        """See IFeed.
        """
        syntool = getToolByName(self, 'portal_syndication')
        return syntool.getUpdatePeriod()

    def getUpdateFrequency(self):
        """See IFeed.
        """
        syntool = getToolByName(self, 'portal_syndication')
        return syntool.getUpdateFrequency()

    def getImageURL(self):
        """See IFeed.
        """
        portal_url = getToolByName(self, 'portal_url')()
        return '%s/logo.png' % portal_url

    def getEncoding(self):
        """See IFeed.
        """
        pp = getToolByName(self, 'portal_properties')
        return pp.site_properties.getProperty('default_charset')

    def getMaxEntries(self):
        """See IFeed.
        """
        syntool = getToolByName(self, 'portal_syndication')
        return syntool.getMaxEntries()

    def getBaseURL(self):
        """See IFeed.
        """
        return getToolByName(self, 'portal_url')()

    def getModifiedDate(self):
        """See IFeed.
        """
        modified = DateTime()
        for feedentry in self.getFeedEntries():
            fe_modified = feedentry.getModifiedDate()
            if fe_modified < modified:
                modified = fe_modified
        return modified



class SyndicationFeed(FeedMixin, BaseFolder):
    """An implementation of ISyndicationManager
    """

    implements(IFeed)

    security = ClassSecurityInfo()
    schema = syndication_feed_schema
    meta_type = portal_type = archetype_name = FEED_PORTAL_TYPE
    global_allow = False
    allowed_content_types = (REFERENCED_FEED_SOURCE_TYPE, REFERENCED_COMMENT_FEED_SOURCE_TYPE)

    security.declareProtected(perms.ViewFeeds, 'getFeedSources')
    def getFeedSources(self):
        """See IFeed"""
        # This next logic should be available built in to contentValues (IMHO)
        obs = self.contentValues()
        feed_sources = []
        for ob in obs:
            if IFeedSource.providedBy(ob):
                feed_sources.append(ob)
        return feed_sources

    def getWebURL(self):
        """See IFeed.
        """
        # XXX This could optionally be overridden to point to a more appropriate URL.
        return self.absolute_url()

    def getTitle(self):
        """See IFeed.
        """
        return self.Title()

    def getDescription(self):
        """See IFeed.
        """
        return self.Description()

    def getUID(self):
        """See IFeed.
        """
        # XXX Ick! This is probably a bad implementation as the UID would
        # change if a different SyndicationFeed instance were used.
        # Perhaps better to use the parent SyndicationManager's UID?
        return self.UID()

    def addReferencedFeedSource(self, content):
        id = self.genId()
        self.invokeFactory(REFERENCED_FEED_SOURCE_TYPE, id)
        rfs = getattr(self, id)
        rfs.setSource(content.UID())
        return rfs

    def genId(self):
        while 1:
            id = str( random.randint(0, 99999) )
            if getattr(self, id, None) is None:
                break
        return id


registerType(SyndicationFeed, PROJECTNAME)
